require 'essential/errors/api_error'

module Essential
  module Client
    module ClassMethods
      def request(method, url, sid: nil, token: nil, params: {}, headers: {})
        # GET -> :get, etc
        method = method.to_s.downcase.to_sym

        sid   ||= self.sid
        token ||= self.token

        uri = URI.join(self. api_base, url)

        headers = {
          'Content-Type' => 'application/json',
          'Accept'       => 'application/json',
          'User-Agent'   => user_agent(),
        }.merge(headers || {})

        case method
        when :get, :delete
          uri.query = URI.encode_www_form(params) if params && params.any?
        else
          payload = params.to_json
        end

        opts = {
          method: method,
          url: uri.to_s,
          user: sid,
          password: token,
          timeout: 10,      # TODO config
          open_timeout: 10, # TODO: config
          headers: headers,
          payload: payload
        }

        if $debug
          puts format('%s %s', method.to_s.upcase, uri)
          puts format('opts: %s', JSON.pretty_generate(opts))
        end

        begin
          response = RestClient::Request.execute(opts)
        rescue StandardError => e
          raise Essential::APIError.from_exception(e)
        end

        response
      end

      protected

      def user_agent
        format('Essential Ruby (%s)', Essential::VERSION)
      end
    end

    def self.included(base)
      base.extend(ClassMethods)
    end

  end
end
