module Essential
  class APIError < StandardError
    private_class_method :new

    attr_reader :http_status, :body, :response
    attr_reader :type, :message, :code, :params

    def self.from_exception(e)
      case e
      # when SocketError
      # when NoMethodError
      when RestClient::ExceptionWithResponse
        new(
          http_status: e.http_code,
          body: e.http_body,
          response: e.response
        )
      when RestClient::Exception
        new(http_status: e.http_code)
      # when Errno::ECONNREFUSED
      else
        raise e
      end
    end

    def initialize(http_status: nil, body: nil, response: nil)
      @http_status = http_status
      @response = response

      if body
        begin
          @body = JSON.parse(body)
          if @body.key?('error')
            error    = @body['error']
            @type    = error['type']
            @message = error['message']
            @code    = error['code']
            @params  = error['params']
          end
        rescue JSON::ParserError
          @body = body
        end
      end
    end

    def to_s
      parts = []
      parts << format('<%s>', self.http_status)
      parts << self.type
      parts << self.message if self.message
      parts << self.params.to_json if self.params
      parts.join(' - ')
    end

  end
end
