module Essential::Messaging
  class Message < Essential::Resource
    extend Essential::Resource::Create
    extend Essential::Resource::List
    include Essential::Resource::Delete

    attr_property :status, :body, :media_urls,
                  :notified_received_at, :notified_received_result,
                  :onstatus_url, :status_updated_at,
                  :notified_status_at, :notified_status_result,
                  :delivery_attempts, :delivery_attempted_at,
                  :msecs_in_flight,
                  :created_at, :updated_at

    # Timestamps, one and all!
    attr_schema [
      :notified_received_at, :status_updated_at, :notified_status_at,
      :delivery_attempted_at, :created_at, :updated_at
    ].reduce({}) {|h,k| h[k] = Time; h}

    # special aliases for create
    attr_unfiltered :subscriber, :transport, :channel

    attr_relation account_sid:    'Essential::Account'
    attr_relation channel_sid:    'Channel'
    attr_relation transport_sid:  'Transport'
    attr_relation subscriber_sid: 'Subscriber'

    alias :redact :delete

    def self.status(params: {}, headers: @headers)
      # permitted:
      # :start_date, :end_date, :carrier, :status, :channel, :channel_sid
      resp = self.request(
        :get,
        url: '/v2/account/messages/analytics/status',
        params: params,
        headers: headers
      )
      JSON.parse(resp)
    end

    def self.queue_status(params: {}, headers: @headers)
      # permitted:
      # :channel, :channel_name
      resp = self.request(
        :get,
        url: '/v2/account/messages/analytics/queue_status',
        params: params,
        headers: headers
      )
      JSON.parse(resp)
    end

    def self.seconds_in_flight(params: {}, headers: @headers)
      # permitted:
      # :start_date, :end_date, :carrier
      resp = self.request(
        :get,
        url: '/v2/account/messages/analytics/seconds_in_flight',
        params: params,
        headers: headers
      )
      JSON.parse(resp)
    end

  end
end
