module Essential::Messaging
  class Subscriber < Essential::Resource
    extend Essential::Resource::Create
    extend Essential::Resource::List
    include Essential::Resource::Update
    include Essential::Resource::Delete

    attr_property :phone_number, :status, :carrier,
                  :subscribed_at, :unsubscribed_at, :created_at, :updated_at

    # special aliases for create
    attr_unfiltered :transport, :channel

    attr_schema subscribed_at: Time, unsubscribed_at: Time, created_at: Time, updated_at: Time

    attr_relation account_sid:   'Essential::Account'
    attr_relation channel_sid:   'Channel'
    attr_relation transport_sid: 'Transport'

    alias :unsubscribe :delete

    def messages
      Essential::Resource::PaginatorProxy.new(
        Message,
        params: {subscriber: self.sid},
        headers: @headers
      )
    end

    def self.by_carrier(params: {}, headers: @headers)
      # permitted:
      # :start_date, :end_date, :carrier
      resp = self.request(
        :get,
        url: '/v2/account/subscribers/analytics/by_carrier',
        params: params,
        headers: headers
      )
      JSON.parse(resp)
    end

    def self.by_transport(params: {}, headers: @headers)
      # permitted:
      # :start_date, :end_date, :carrier
      resp = self.request(
        :get,
        url: '/v2/account/subscribers/analytics/by_transport',
        params: params,
        headers: headers
      )
      JSON.parse(resp)
    end

  end
end
