require 'essential/resource/attr_relations'

module Essential
  class Resource
    module AttrMethods

      module ClassMethods
        include AttrRelations

        def from_attributes(attributes, headers: @headers)
          self.new(attrs: attributes, headers: headers)
        end

        def permitted_attrs
          _permitted_attrs.dup
        end

        def filter_attrs(attributes)
          filtered = {}
          attributes.keys.each do |k|
            if self.permitted_attrs.include?(k.to_sym)
              filtered[k.to_s] = attributes[k]
            end
          end

          filtered
        end

        protected

        def _permitted_attrs
          @permitted_attrs ||= Set[:sid]
        end

        def attr_property(*names)
          attr_unfiltered(*names)
          names.each do |name|
            define_method(name.to_sym) do
              self[name.to_s]
            end
          end
        end

        def attr_unfiltered(*names)
          _permitted_attrs.merge names.map(&:to_sym)
        end

        def attr_schema(pairs)
          @schema ||= {}
          @schema = @schema.merge(Hash[pairs.map{|k,v| [k.to_s, v]}])
        end

      end

      def filter_attrs(attributes)
        self.class.filter_attrs(attributes)
      end

      def self.included(base)
        base.extend(ClassMethods)
      end

    end
  end
end
